@@ -1,7 +1,9 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# Copyright (C) 2023 Fortra. All rights reserved.
+# Copyright Fortra, LLC and its affiliated companies 
+#
+# All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -102,7 +104,7 @@ class SamrObject:
 
     def _get_user_handle(self, domain_handle, username):
         user_rid = self._get_object_rid(domain_handle, username)
-        response = samr.hSamrOpenUser(self._dce, domain_handle, samr.USER_ALL_ACCESS, user_rid)
+        response = samr.hSamrOpenUser(self._dce, domain_handle, samr.MAXIMUM_ALLOWED, user_rid)
         return response['UserHandle']
 
     def _get_group_handle(self, domain_handle, alias_name):
@@ -357,12 +359,12 @@ class Net:
             print("[+] {} account deleted succesfully!".format(self.__action))
 
         elif self.__is_option_present(self.__options, 'join'):
-            print("[*] Adding user account '{}' to group '{}'".format(self.__options.name, self.__options.join))
+            print("[*] Adding user account '{}' to group '{}'".format(self.__options.join,self.__options.name))
             actionObject.Join(self.__options.name, self.__options.join)
             print("[+] User account added to {} succesfully!".format(self.__options.name))
 
         elif self.__is_option_present(self.__options, 'unjoin'):
-            print("[*] Removing user account '{}' from group '{}'".format(self.__options.name, self.__options.unjoin))
+            print("[*] Removing user account '{}' from group '{}'".format(self.__options.unjoin,self.__options.name))
             actionObject.UnJoin(self.__options.name, self.__options.unjoin)
             print("[+] User account removed from {} succesfully!".format(self.__options.name))
 
@@ -379,7 +381,7 @@ class Net:
                 print("Comment".ljust(30), info['AdminComment'])
                 print("User's comment".ljust(30), info['UserComment'])
                 print("Country/region code".ljust(30), "000 (System Default)" if info['CountryCode'] == 0 else info['CountryCode'])
-                print("Account active".ljust(30), self.__b2s(info['WhichFields'] & samr.USER_ACCOUNT_DISABLED == samr.USER_ACCOUNT_DISABLED))
+                print("Account active".ljust(30), self.__b2s(info['UserAccountControl'] & samr.USER_ACCOUNT_DISABLED != samr.USER_ACCOUNT_DISABLED))
                 print("Account expires".ljust(30), self.__get_time_string(info['AccountExpires']))
                 print('')
                 print("Password last set".ljust(30), self.__get_time_string(info['PasswordLastSet']))
