@@ -1,7 +1,9 @@
 #!/usr/bin/env python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# Copyright (C) 2023 Fortra. All rights reserved.
+# Copyright Fortra, LLC and its affiliated companies 
+#
+# All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -33,6 +35,7 @@ import codecs
 import logging
 import sys
 import time
+import binascii
 from struct import unpack
 
 from impacket import version
@@ -41,7 +44,7 @@ from impacket.examples import logger
 from impacket.examples.utils import parse_target
 from impacket.system_errors import ERROR_NO_MORE_ITEMS
 from impacket.structure import hexdump
-from impacket.smbconnection import SMBConnection
+from impacket.smbconnection import SMBConnection, SessionError
 from impacket.dcerpc.v5.dtypes import READ_CONTROL
 
 
@@ -173,7 +176,8 @@ class RegHandler:
             self.__remoteOps.enableRegistry()
         except Exception as e:
             logging.debug(str(e))
-            logging.warning('Cannot check RemoteRegistry status. Hoping it is started...')
+            logging.warning('Cannot check RemoteRegistry status. Triggering start trough named pipe...')
+            self.triggerWinReg()
             self.__remoteOps.connectWinReg()
 
         try:
@@ -200,6 +204,17 @@ class RegHandler:
             if self.__remoteOps:
                 self.__remoteOps.finish()
 
+    def triggerWinReg(self):
+        # original idea from https://twitter.com/splinter_code/status/1715876413474025704
+        tid = self.__smbConnection.connectTree('IPC$')
+        try:
+            self.__smbConnection.openFile(tid, r'\winreg', 0x12019f, creationOption=0x40, fileAttributes=0x80)
+        except SessionError:
+            # STATUS_PIPE_NOT_AVAILABLE error is expected
+            pass
+        # give remote registry time to start
+        time.sleep(1)
+
     def save(self, dce, keyName):
         hRootKey, subKey = self.__strip_root_key(dce, keyName)
         outputFileName = "%s\%s.save" % (self.__options.outputPath, subKey)
@@ -278,25 +293,36 @@ class RegHandler:
                 raise Exception('Error parsing value type %s' % self.__options.vt)
 
             #Fix (?) for packValue function
-            if dwType in (
-                rrp.REG_DWORD, rrp.REG_DWORD_BIG_ENDIAN, rrp.REG_DWORD_LITTLE_ENDIAN,
-                rrp.REG_QWORD, rrp.REG_QWORD_LITTLE_ENDIAN
-            ):
-                valueData = int(self.__options.vd)
+            if dwType == rrp.REG_MULTI_SZ:
+                vd = '\0'.join(self.__options.vd)
+                valueData = vd + 2 * '\0' # REG_MULTI_SZ ends with 2 null-bytes
+                valueDataToPrint = vd.replace('\0', '\n\t\t')
             else:
-                valueData = self.__options.vd
+                vd = self.__options.vd[0] if len(self.__options.vd) > 0 else ''
+                if dwType in (
+                    rrp.REG_DWORD, rrp.REG_DWORD_BIG_ENDIAN, rrp.REG_DWORD_LITTLE_ENDIAN,
+                    rrp.REG_QWORD, rrp.REG_QWORD_LITTLE_ENDIAN
+                ):
+                    valueData = int(vd)
+                elif dwType == rrp.REG_BINARY:
+                    bin_value_len = len(vd)
+                    bin_value_len += (bin_value_len & 1)
+                    valueData = binascii.a2b_hex(vd.ljust(bin_value_len, '0'))
+                else:
+                    valueData = vd + "\0" # Add a NULL Byte as terminator for Non Binary values
+                valueDataToPrint = valueData
 
             ans3 = rrp.hBaseRegSetValue(
                 dce, ans2['phkResult'], self.__options.v, dwType, valueData
             )
 
             if ans3['ErrorCode'] == 0:
-                print('Successfully set key %s\\%s of type %s to value %s' % (
-                    keyName, self.__options.v, self.__options.vt, valueData
+                print('Successfully set\n\tkey\t%s\\%s\n\ttype\t%s\n\tvalue\t%s' % (
+                    keyName, self.__options.v, self.__options.vt, valueDataToPrint
                 ))
             else:
-                print('Error 0x%08x while setting key %s\\%s of type %s to value %s' % (
-                    ans3['ErrorCode'], keyName, self.__options.v, self.__options.vt, valueData
+                print('Error 0x%08x while setting\n\tkey\t%s\\%s\n\ttype\t%s\n\tvalue\t%s' % (
+                    ans3['ErrorCode'], keyName, self.__options.v, self.__options.vt, valueDataToPrint
                 ))
 
     def delete(self, dce, keyName):
@@ -413,8 +439,10 @@ class RegHandler:
             raise Exception('Error parsing keyName %s' % keyName)
         if rootKey.upper() == 'HKLM':
             ans = rrp.hOpenLocalMachine(dce)
-        elif rootKey.upper() == 'HKU':
+        elif rootKey.upper() == 'HKCU':
             ans = rrp.hOpenCurrentUser(dce)
+        elif rootKey.upper() == 'HKU':
+            ans = rrp.hOpenUsers(dce)
         elif rootKey.upper() == 'HKCR':
             ans = rrp.hOpenClassesRoot(dce)
         else:
@@ -520,7 +548,7 @@ if __name__ == '__main__':
     query_parser.add_argument('-keyName', action='store', required=True,
                               help='Specifies the full path of the subkey. The '
                                    'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
-                                   ' HKU, HKCR.')
+                                   ' HKU, HKCU, HKCR.')
     query_parser.add_argument('-v', action='store', metavar="VALUENAME", required=False, help='Specifies the registry '
                            'value name that is to be queried. If omitted, all value names for keyName are returned. ')
     query_parser.add_argument('-ve', action='store_true', default=False, required=False, help='Queries for the default '
@@ -533,22 +561,23 @@ if __name__ == '__main__':
     add_parser.add_argument('-keyName', action='store', required=True,
                               help='Specifies the full path of the subkey. The '
                                    'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
-                                   ' HKU, HKCR.')
+                                   ' HKU, HKCU, HKCR.')
     add_parser.add_argument('-v', action='store', metavar="VALUENAME", required=False, help='Specifies the registry '
-                           'value name that is to be set.')
+                           'value name that is to be set. Set to "" to write the (Defualt) value')
     add_parser.add_argument('-vt', action='store', metavar="VALUETYPE", required=False, help='Specifies the registry '
                            'type name that is to be set. Default is REG_SZ. Valid types are: REG_NONE, REG_SZ, REG_EXPAND_SZ, '
                            'REG_BINARY, REG_DWORD, REG_DWORD_BIG_ENDIAN, REG_LINK, REG_MULTI_SZ, REG_QWORD',
                             default='REG_SZ')
-    add_parser.add_argument('-vd', action='store', metavar="VALUEDATA", required=False, help='Specifies the registry '
-                           'value data that is to be set.', default='')
+    add_parser.add_argument('-vd', action='append', metavar="VALUEDATA", required=False, help='Specifies the registry '
+                           'value data that is to be set. In case of adding a REG_MULTI_SZ value, set this option once for each '
+                           'line you want to add.', default=[])
 
     # An delete command
     delete_parser = subparsers.add_parser('delete', help='Deletes a subkey or entries from the registry')
     delete_parser.add_argument('-keyName', action='store', required=True,
                               help='Specifies the full path of the subkey. The '
                                    'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
-                                   ' HKU, HKCR.')
+                                   ' HKU, HKCU, HKCR.')
     delete_parser.add_argument('-v', action='store', metavar="VALUENAME", required=False, help='Specifies the registry '
                            'value name that is to be deleted.')
     delete_parser.add_argument('-va', action='store_true', required=False, help='Delete all values under this key.')
@@ -564,7 +593,7 @@ if __name__ == '__main__':
     save_parser.add_argument('-keyName', action='store', required=True,
                                help='Specifies the full path of the subkey. The '
                                     'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
-                                    ' HKU, HKCR.')
+                                    ' HKU, HKCU, HKCR.')
     save_parser.add_argument('-o', dest='outputPath', action='store', metavar='\\\\192.168.0.2\share', required=True, help='Output UNC path the target system must export the registry saves to')
 
     # A special backup command to save HKLM\SAM, HKLM\SYSTEM and HKLM\SECURITY
